import { Directive, Input, ContentChild, EventEmitter, Component, Inject, forwardRef, Output, ViewEncapsulation, ContentChildren, ViewChildren, HostListener, Pipe, NgModule } from '@angular/core';
import { CommonModule } from '@angular/common';
import { FormsModule } from '@angular/forms';
import { Subscription } from 'rxjs';

const drag = (event, { move, up }) => {
    const { pageX, pageY } = event;
    let x = pageX;
    let y = pageY;
    let moved = false;
    const mouseMoveHandler = (e) => {
        // tslint:disable-next-line:no-shadowed-variable
        const { pageX, pageY } = e;
        const dx = pageX - x;
        const dy = pageY - y;
        x = pageX;
        y = pageY;
        if (dx || dy) {
            moved = true;
        }
        move(e, dx, dy, x, y);
        e.preventDefault(); // to avoid text selection
    };
    const mouseUpHandler = (e) => {
        // tslint:disable-next-line:no-shadowed-variable
        const { pageX, pageY } = e;
        x = pageX;
        y = pageY;
        document.removeEventListener('mousemove', mouseMoveHandler);
        document.removeEventListener('mouseup', mouseUpHandler);
        if (up) {
            up(e, x, y, moved);
        }
    };
    document.addEventListener('mousemove', mouseMoveHandler);
    document.addEventListener('mouseup', mouseUpHandler);
};

const defaultTranslations = {
    indexColumn: 'index',
    selectColumn: 'select',
    expandColumn: 'expand',
    paginationLimit: 'Limit',
    paginationRange: 'Results'
};

class DataTableColumnDirective {
    constructor() {
        this.ngStyleClass = {};
        this.sortable = false;
        this.resizable = false;
        this.visible = true;
    }
    get cellColor() {
        if (this.cellColors !== undefined) {
            return this.cellColors(this);
        }
    }
    ngOnInit() {
        if (!this.styleClass && this.property) {
            if (/^[a-zA-Z0-9_]+$/.test(this.property)) {
                this.styleClass = 'column-' + this.property;
            }
            else {
                this.styleClass = 'column-' + this.property.replace(/[^a-zA-Z0-9_]/g, '');
            }
        }
        if (this.styleClass != null) {
            this.ngStyleClass = {
                [this.styleClass]: true
            };
        }
    }
}
DataTableColumnDirective.decorators = [
    { type: Directive, args: [{
                selector: 'data-table-column'
            },] }
];
DataTableColumnDirective.propDecorators = {
    header: [{ type: Input }],
    sortable: [{ type: Input }],
    resizable: [{ type: Input }],
    property: [{ type: Input }],
    styleClass: [{ type: Input }],
    cellColors: [{ type: Input }],
    width: [{ type: Input }],
    visible: [{ type: Input }],
    cellTemplate: [{ type: ContentChild, args: ['cellTemplate',] }],
    headerTemplate: [{ type: ContentChild, args: ['headerTemplate',] }]
};

class DataTableRowComponent {
    constructor(dataTable) {
        this.dataTable = dataTable;
        // tslint:disable-next-line:variable-name
        this._selected = false;
        this.expanded = false;
        this.selectedChange = new EventEmitter();
        // FIXME is there no template keyword for this in angular 2?
        this._this = this;
    }
    get selected() {
        return this._selected;
    }
    set selected(selected) {
        this._selected = selected;
        this.selectedChange.emit(selected);
    }
    // other:
    get displayIndex() {
        if (this.dataTable.pagination) {
            return this.dataTable.displayParams.offset + this.index + 1;
        }
        else {
            return this.index + 1;
        }
    }
    getTooltip() {
        if (this.dataTable.rowTooltip) {
            return this.dataTable.rowTooltip(this);
        }
        return '';
    }
    populateCell(column) {
        column.item = this.item;
        column.index = this.index;
        column.selected = this.selected;
        return column;
    }
    ngOnDestroy() {
        this.selected = false;
    }
}
DataTableRowComponent.decorators = [
    { type: Component, args: [{
                selector: '[dataTableRow]',
                template: "<tr class=\"data-table-row\"\n    [title]=\"getTooltip()\"\n    [style.background-color]=\"dataTable.getRowColor(_this)\"\n    [class.row-odd]=\"index % 2 === 0\"\n    [class.row-even]=\"index % 2 === 1\"\n    [class.selected]=\"selected\"\n    [class.clickable]=\"dataTable.selectOnRowClick\"\n    (dblclick)=\"dataTable.rowDoubleClicked(_this, $event)\"\n    (click)=\"dataTable.rowClicked(_this, $event)\">\n  <td *ngIf=\"dataTable.expandColumnVisible\" class=\"row-expand-button\"\n      (click)=\"expanded = !expanded; $event.stopPropagation(); dataTable.rowExpanded(_this, $event)\">\n    <span class=\"fa {{expanded ? 'fa-arrow-circle-o-down' : 'fa-arrow-circle-o-right'}}\"></span>\n  </td>\n  <td *ngIf=\"dataTable.indexColumnVisible\" class=\"index-column\" [textContent]=\"displayIndex\"></td>\n  <td *ngIf=\"dataTable.selectColumnVisible\" class=\"select-column\">\n    <input type=\"checkbox\" [(ngModel)]=\"selected\"/>\n  </td>\n  <ng-container *ngFor=\"let column of dataTable.columns\">\n    <td class=\"data-column\"\n        [ngClass]=\"column.ngStyleClass\"\n        [style.background-color]=\"column.cellColor\"\n        (click)=\"dataTable.cellClicked(populateCell(column), $event)\"\n        *ngIf=\"column.visible\">\n      <div *ngIf=\"!column.cellTemplate\" [textContent]=\"item[column.property]\"></div>\n      <ng-container *ngIf=\"column.cellTemplate\" [ngTemplateOutlet]=\"column.cellTemplate\"\n                    [ngTemplateOutletContext]=\"{column: column, row: _this, item: item}\"></ng-container>\n    </td>\n  </ng-container>\n</tr>\n<tr class=\"row-expansion {{expanded? '' : 'd-none'}}\"\n    *ngIf=\"dataTable.expandableRows && expanded\">\n  <td [attr.colspan]=\"dataTable.columnCount\">\n    <ng-container [ngTemplateOutlet]=\"dataTable.expandTemplate\"\n                  [ngTemplateOutletContext]=\"{row: _this, item: item}\"></ng-container>\n  </td>\n</tr>\n",
                styles: [".select-column{text-align:center}.row-expand-button{cursor:pointer;text-align:center}.clickable{cursor:pointer}"]
            },] }
];
DataTableRowComponent.ctorParameters = () => [
    { type: DataTableComponent, decorators: [{ type: Inject, args: [forwardRef(() => DataTableComponent),] }] }
];
DataTableRowComponent.propDecorators = {
    item: [{ type: Input }],
    index: [{ type: Input }],
    selectedChange: [{ type: Output }]
};

class DataTableTitleDirective {
    constructor() {
        this.title = '';
        this.controls = true;
    }
}
DataTableTitleDirective.decorators = [
    { type: Directive, args: [{
                selector: 'data-table-title'
            },] }
];
DataTableTitleDirective.propDecorators = {
    title: [{ type: Input }],
    titleTemplate: [{ type: ContentChild, args: ['titleTemplate',] }],
    controls: [{ type: Input }]
};

class DataTableComponent {
    constructor() {
        this._sortAsc = true;
        this._offset = 0;
        this._limit = 10;
        this._items = [];
        this._scheduledReload = null;
        this._selectAllCheckbox = false;
        this._reloading = false;
        this._resizeInProgress = false;
        this.subscription = new Subscription();
        this.selectedRows = [];
        this.resizeLimit = 30;
        this.pagination = true;
        this.indexColumn = true;
        this.indexColumnHeader = '';
        this.selectColumn = false;
        this.multiSelect = true;
        this.substituteRows = true;
        this.expandableRows = false;
        this.translations = defaultTranslations;
        this.selectOnRowClick = false;
        this.autoReload = true;
        this.showReloading = false;
        // event handlers:
        this.rowClick = new EventEmitter();
        this.rowExpand = new EventEmitter();
        this.rowDoubleClick = new EventEmitter();
        this.headerClick = new EventEmitter();
        this.cellClick = new EventEmitter();
        this.reload = new EventEmitter();
    }
    get items() {
        return this._items;
    }
    set items(items) {
        this._items = items;
        this.onReloadFinished();
    }
    get sortBy() {
        return this._sortBy;
    }
    set sortBy(value) {
        this._sortBy = value;
        this.triggerReload();
    }
    get sortAsc() {
        return this._sortAsc;
    }
    set sortAsc(value) {
        this._sortAsc = value;
        this.triggerReload();
    }
    get offset() {
        return this._offset;
    }
    set offset(value) {
        this._offset = value;
        this.triggerReload();
    }
    get limit() {
        return this._limit;
    }
    set limit(value) {
        this._limit = value;
        this.triggerReload();
    }
    // calculated property:
    get page() {
        return Math.floor(this.offset / this.limit) + 1;
    }
    set page(value) {
        this.offset = (value - 1) * this.limit;
    }
    get lastPage() {
        return Math.ceil(this.itemCount / this.limit);
    }
    get reloading() {
        return this._reloading;
    }
    get displayParams() {
        return {
            sortBy: this.sortBy,
            sortAsc: this.sortAsc,
            offset: this.offset,
            limit: this.limit
        };
    }
    get selectAllCheckbox() {
        return this._selectAllCheckbox;
    }
    set selectAllCheckbox(value) {
        this._selectAllCheckbox = value;
        this.onSelectAllChanged(value);
    }
    get columnCount() {
        let count = 0;
        count += this.indexColumnVisible ? 1 : 0;
        count += this.selectColumnVisible ? 1 : 0;
        count += this.expandColumnVisible ? 1 : 0;
        this.columns.toArray().forEach(column => {
            count += column.visible ? 1 : 0;
        });
        return count;
    }
    get substituteItems() {
        return Array.from({ length: this.displayParams.limit - this.items.length });
    }
    getRowColor(row) {
        if (this.rowColors !== undefined) {
            return this.rowColors(row);
        }
    }
    // setting multiple observable properties simultaneously
    sort(sortBy, asc) {
        this.sortBy = sortBy;
        this.sortAsc = asc;
    }
    reloadItems() {
        this._reloading = true;
        this.reload.emit(this._getRemoteParameters());
    }
    rowClicked(row, event) {
        this.rowClick.emit({ row, event });
    }
    rowDoubleClicked(row, event) {
        this.rowDoubleClick.emit({ row, event });
    }
    headerClicked(column, event) {
        if (!this._resizeInProgress) {
            this.headerClick.emit({ column, event });
        }
        else {
            // this is because I can't prevent click from mouseup of the drag end
            this._resizeInProgress = false;
        }
    }
    cellClicked(cell, event) {
        this.cellClick.emit({ cell, event });
    }
    rowExpanded(row, event) {
        this.rowExpand.emit({ row, event });
    }
    onRowSelectChanged(row) {
        // maintain the selectedRow(s) view
        if (this.multiSelect) {
            const index = this.selectedRows.indexOf(row);
            if (row.selected && index < 0) {
                this.selectedRows.push(row);
            }
            else if (!row.selected && index >= 0) {
                this.selectedRows.splice(index, 1);
            }
        }
        else {
            if (row.selected) {
                this.selectedRow = row;
            }
            else if (this.selectedRow === row) {
                this.selectedRow = undefined;
            }
        }
        // unselect all other rows:
        if (row.selected && !this.multiSelect) {
            this.rows.filter(r => r.selected).forEach(r => {
                if (r !== row) { // avoid endless loop
                    r.selected = false;
                }
            });
        }
    }
    resizeColumnStart(event, column, columnElement) {
        this._resizeInProgress = true;
        drag(event, {
            move: (moveEvent, dx) => {
                if (this.isResizeInLimit(columnElement, dx)) {
                    column.width = columnElement.offsetWidth + dx;
                }
            },
        });
    }
    ngOnInit() {
        this.indexColumnVisible = this.indexColumn;
        this.selectColumnVisible = this.selectColumn;
        this.expandColumnVisible = this.expandableRows;
        this.subscription.add(this.headerClick.subscribe(tableEvent => this.sortColumn(tableEvent.column)));
        if (this.selectOnRowClick) {
            this.subscription.add(this.rowClick.subscribe(tableEvent => tableEvent.row.selected = !tableEvent.row.selected));
        }
        if (this.autoReload && this._scheduledReload == null) {
            this.reloadItems();
        }
    }
    ngOnDestroy() {
        this.subscription.unsubscribe();
        this.subscription = undefined;
    }
    onReloadFinished() {
        this._selectAllCheckbox = false;
        this._reloading = false;
    }
    triggerReload() {
        // for avoiding cascading reloads if multiple params are set at once:
        if (this._scheduledReload) {
            clearTimeout(this._scheduledReload);
        }
        this._scheduledReload = setTimeout(() => {
            this.reloadItems();
        });
    }
    _getRemoteParameters() {
        const params = {};
        if (this.sortBy) {
            params.sortBy = this.sortBy;
            params.sortAsc = this.sortAsc;
        }
        if (this.pagination) {
            params.offset = this.offset;
            params.limit = this.limit;
        }
        return params;
    }
    sortColumn(column) {
        if (column.sortable) {
            const ascending = this.sortBy === column.property ? !this.sortAsc : true;
            this.sort(column.property, ascending);
        }
    }
    onSelectAllChanged(value) {
        this.rows.toArray().forEach(row => row.selected = value);
    }
    isResizeInLimit(columnElement, dx) {
        /* This is needed because CSS min-width didn't work on table-layout: fixed.
         Without the limits, resizing can make the next column disappear completely,
         and even increase the table width. The current implementation suffers from the fact,
         that offsetWidth sometimes contains out-of-date values. */
        return !((dx < 0 && (columnElement.offsetWidth + dx) <= this.resizeLimit) ||
            !columnElement.nextElementSibling || // resizing doesn't make sense for the last visible column
            (dx >= 0 && (columnElement.nextElementSibling.offsetWidth + dx) <= this.resizeLimit));
    }
}
DataTableComponent.decorators = [
    { type: Component, args: [{
                selector: 'data-table',
                template: "<data-table-title *ngIf=\"title\"></data-table-title>\n<div class=\"position-relative\">\n  <table class=\"table table-condensed data-table\">\n    <thead>\n    <tr>\n      <th class=\"expand-column-header\" *ngIf=\"expandColumnVisible\">\n      <th class=\"index-column-header\" *ngIf=\"indexColumnVisible\">\n        <span [textContent]=\"indexColumnHeader\"></span>\n      </th>\n      <th class=\"select-column-header\" *ngIf=\"selectColumnVisible\">\n        <input type=\"checkbox\" [(ngModel)]=\"selectAllCheckbox\" *ngIf=\"multiSelect\"/>\n      </th>\n      <ng-container *ngFor=\"let column of columns\">\n        <th class=\"column-header\"\n            #th\n            (click)=\"headerClicked(column, $event)\"\n            [class.sortable]=\"column.sortable\"\n            [class.resizable]=\"column.resizable\"\n            [ngClass]=\"column.ngStyleClass\"\n            [style.width]=\"column.width | px\"\n            *ngIf=\"column.visible\">\n          <span *ngIf=\"!column.headerTemplate\" [textContent]=\"column.header\"></span>\n          <ng-container *ngIf=\"column.headerTemplate\" [ngTemplateOutlet]=\"column.headerTemplate\"\n                        [ngTemplateOutletContext]=\"{column: column}\"></ng-container>\n          <span class=\"column-sort-icon\" *ngIf=\"column.sortable\">\n            <i class=\"fa fa-sort column-sortable-icon\" *ngIf=\"column.property !== sortBy else sortByTemplate\"></i>\n            <ng-template #sortByTemplate>\n              <i class=\"fa {{sortAsc ? 'fa-sort-desc' : 'fa-sort-asc'}}\"></i>\n            </ng-template>\n          </span>\n          <span *ngIf=\"column.resizable\" class=\"column-resize-handle\"\n                (mousedown)=\"resizeColumnStart($event, column, th)\"></span>\n        </th>\n      </ng-container>\n    </tr>\n    </thead>\n    <tbody #row\n           dataTableRow\n           [item]=\"item\"\n           [index]=\"index\"\n           (selectedChange)=\"onRowSelectChanged(row)\"\n           *ngFor=\"let item of items; let index=index\">\n    </tbody>\n    <tbody *ngIf=\"pagination && substituteRows\">\n      <tr\n        [class.row-odd]=\"(index + items.length) % 2 === 0\"\n        [class.row-even]=\"(index + items.length) % 2 === 1\"\n        *ngFor=\"let item of substituteItems, let index = index\">\n        <td *ngIf=\"expandColumnVisible\"></td>\n        <td *ngIf=\"indexColumnVisible\">&nbsp;</td>\n        <td *ngIf=\"selectColumnVisible\"></td>\n        <ng-container *ngFor=\"let column of columns\">\n          <td *ngIf=\"column.visible\">\n        </ng-container>\n      </tr>\n    </tbody>\n  </table>\n  <div class=\"loading-cover\" *ngIf=\"showReloading && reloading\"></div>\n</div>\n<data-table-pagination *ngIf=\"pagination\"></data-table-pagination>\n",
                encapsulation: ViewEncapsulation.None,
                styles: [".data-table{box-shadow:0 0 15px #ececec;table-layout:fixed}.data-table td{vertical-align:middle}.data-table tbody>tr>td,.data-table thead>tr>th{overflow:hidden}.data-table>tbody+tbody{border-top:none}.data-table .row-odd{background-color:#f6f6f6}.data-table .expand-column-header{width:50px}.data-table .select-column-header{text-align:center;width:50px}.data-table .index-column-header{width:40px}.data-table .column-header{position:relative}.data-table .column-header.sortable{cursor:pointer}.data-table .column-header.resizable{margin-right:8px}.data-table .column-header .column-sort-icon{float:right;margin-right:8px}.data-table .column-header .column-sort-icon .column-sortable-icon{color:#d3d3d3}.data-table .column-header .column-resize-handle{cursor:col-resize;height:100%;margin:0;padding:0;position:absolute;right:0;top:0;width:8px}.loading-cover{background-color:hsla(0,0%,100%,.3);height:100%;position:absolute;top:0;width:100%}"]
            },] }
];
DataTableComponent.propDecorators = {
    itemCount: [{ type: Input }],
    pagination: [{ type: Input }],
    indexColumn: [{ type: Input }],
    indexColumnHeader: [{ type: Input }],
    rowColors: [{ type: Input }],
    rowTooltip: [{ type: Input }],
    selectColumn: [{ type: Input }],
    multiSelect: [{ type: Input }],
    substituteRows: [{ type: Input }],
    expandableRows: [{ type: Input }],
    translations: [{ type: Input }],
    selectOnRowClick: [{ type: Input }],
    autoReload: [{ type: Input }],
    showReloading: [{ type: Input }],
    rowClick: [{ type: Output }],
    rowExpand: [{ type: Output }],
    rowDoubleClick: [{ type: Output }],
    headerClick: [{ type: Output }],
    cellClick: [{ type: Output }],
    reload: [{ type: Output }],
    title: [{ type: ContentChild, args: [forwardRef(() => DataTableTitleDirective),] }],
    columns: [{ type: ContentChildren, args: [DataTableColumnDirective,] }],
    rows: [{ type: ViewChildren, args: [DataTableRowComponent,] }],
    expandTemplate: [{ type: ContentChild, args: ['expandTemplate',] }],
    items: [{ type: Input }],
    sortBy: [{ type: Input }],
    sortAsc: [{ type: Input }],
    offset: [{ type: Input }],
    limit: [{ type: Input }],
    page: [{ type: Input }]
};

class DataTablePaginationComponent {
    constructor(dataTable) {
        this.dataTable = dataTable;
    }
    get maxPage() {
        return Math.ceil(this.dataTable.itemCount / this.dataTable.limit);
    }
    get limit() {
        return this.dataTable.limit;
    }
    set limit(value) {
        if (Number(value) > 0) {
            this.dataTable.limit = Math.floor(+value);
        }
    }
    get page() {
        return this.dataTable.page;
    }
    set page(value) {
        if (Number(value) > 0) {
            this.dataTable.page = Math.floor(+value);
        }
    }
    pageBack() {
        this.dataTable.offset -= Math.min(this.dataTable.limit, this.dataTable.offset);
    }
    pageForward() {
        this.dataTable.offset += this.dataTable.limit;
    }
    pageFirst() {
        this.dataTable.offset = 0;
    }
    pageLast() {
        this.dataTable.offset = (this.maxPage - 1) * this.dataTable.limit;
    }
    keyPress(event) {
        if (!(Number(event.key) >= 0)) {
            event.preventDefault();
        }
    }
}
DataTablePaginationComponent.decorators = [
    { type: Component, args: [{
                selector: 'data-table-pagination',
                template: "<div class=\"navbar p-0 pagination-component\">\n  <div>\n    {{dataTable.translations.paginationRange}}:&nbsp;{{dataTable.offset < 0 ? 0 : dataTable.offset + 1}}&nbsp;-\n    {{[dataTable.offset + dataTable.limit , dataTable.itemCount] | min}}&nbsp;/&nbsp;{{dataTable.itemCount}}\n  </div>\n  <ul class=\"nav justify-content-end\">\n    <li class=\"nav-item pagination-limit\">\n      <div class=\"input-group\">\n        <div class=\"input-group-prepend\">\n          <span class=\"input-group-text\">{{dataTable.translations.paginationLimit}}:</span>\n        </div>\n        <input #limitInput type=\"number\" class=\"form-control\" min=\"1\" step=\"1\"\n               [ngModel]=\"limit\" (blur)=\"limit = limitInput.value\"\n               (keyup.enter)=\"limit = limitInput.value\"\n               (keyup.esc)=\"limitInput.value = limit\"\n               (keypress)=\"keyPress($event)\"/>\n      </div>\n    </li>\n    <li class=\"nav-item\">\n      <button [disabled]=\"dataTable.offset <= 0\" (click)=\"pageFirst()\" class=\"btn btn-default\">&laquo;</button>\n      <button [disabled]=\"dataTable.offset <= 0\" (click)=\"pageBack()\" class=\"btn btn-default\">&lsaquo;</button>\n      <div class=\"d-inline-block\">\n        <div class=\"input-group\">\n          <input #pageInput type=\"number\" class=\"form-control\" min=\"1\" step=\"1\" max=\"{{maxPage}}\"\n                 [ngModel]=\"page\" (blur)=\"page = pageInput.value\"\n                 (keyup.enter)=\"page = pageInput.value\" (keyup.esc)=\"pageInput.value = page\"\n                 (keypress)=\"keyPress($event)\"/>\n          <div class=\"input-group-append\">\n            <span class=\"input-group-text\">/&nbsp;{{dataTable.lastPage}}</span>\n          </div>\n        </div>\n      </div>\n      <button [disabled]=\"(dataTable.offset + dataTable.limit) >= dataTable.itemCount\" (click)=\"pageForward()\"\n              class=\"btn btn-default\">&rsaquo;\n      </button>\n      <button [disabled]=\"(dataTable.offset + dataTable.limit) >= dataTable.itemCount\" (click)=\"pageLast()\"\n              class=\"btn btn-default\">&raquo;\n      </button>\n    </li>\n  </ul>\n</div>\n",
                styles: [".pagination-component .pagination-limit{margin-right:1rem}.pagination-component .pagination-limit input{width:75px}.pagination-component .btn{margin-top:-3px}"]
            },] }
];
DataTablePaginationComponent.ctorParameters = () => [
    { type: DataTableComponent, decorators: [{ type: Inject, args: [forwardRef(() => DataTableComponent),] }] }
];

class DataTableTitleComponent {
    constructor(dataTable) {
        this.dataTable = dataTable;
        this.columnSelectorOpen = false;
    }
    closeSelector() {
        this.columnSelectorOpen = false;
    }
}
DataTableTitleComponent.decorators = [
    { type: Component, args: [{
                selector: 'data-table-title',
                template: "<div class=\"navbar\">\n  <h4 class=\"title\" [textContent]=\"dataTable.title.title\"></h4>\n  <ng-container [ngTemplateOutlet]=\"dataTable.title.titleTemplate\"\n                [ngTemplateOutletContext]=\"{items: dataTable.items}\"\n                *ngIf=\"dataTable.title.titleTemplate\">\n  </ng-container>\n  <div *ngIf=\"dataTable.title.controls\">\n    <button type=\"button\" class=\"btn btn-default btn-sm refresh-button\"\n            (click)=\"dataTable.reloadItems()\">\n      <i class=\"fa fa-refresh\"></i>\n    </button>\n    <button type=\"button\" class=\"btn btn-default btn-sm column-selector-button\"\n            [class.active]=\"columnSelectorOpen\"\n            (click)=\"columnSelectorOpen = !columnSelectorOpen; $event.stopPropagation()\">\n      <i class=\"fa fa-list\"></i>\n    </button>\n    <div class=\"column-selector-wrapper\" (click)=\"$event.stopPropagation()\">\n      <div *ngIf=\"columnSelectorOpen\" class=\"column-selector-box card\">\n        <div *ngIf=\"dataTable.expandableRows\" class=\"column-selector-fixed-column checkbox\">\n            <input type=\"checkbox\" [(ngModel)]=\"dataTable.expandColumnVisible\"/>\n            <span>&nbsp;{{dataTable.translations.expandColumn}}</span>\n        </div>\n        <div *ngIf=\"dataTable.indexColumn\" class=\"column-selector-fixed-column checkbox\">\n            <input type=\"checkbox\" [(ngModel)]=\"dataTable.indexColumnVisible\"/>\n            <span>&nbsp;{{dataTable.translations.indexColumn}}</span>\n        </div>\n        <div *ngIf=\"dataTable.selectColumn\" class=\"column-selector-fixed-column checkbox\">\n            <input type=\"checkbox\" [(ngModel)]=\"dataTable.selectColumnVisible\"/>\n            <span>&nbsp;{{dataTable.translations.selectColumn}}</span>\n        </div>\n        <div *ngFor=\"let column of dataTable.columns\" class=\"column-selector-column checkbox\">\n            <input type=\"checkbox\" [(ngModel)]=\"column.visible\"/>\n            <span>&nbsp;{{column.header}}</span>\n        </div>\n      </div>\n    </div>\n  </div>\n</div>\n",
                styles: [".navbar{padding:10px 0}.navbar .title{margin:5px 0 0 5px}.navbar .column-selector-wrapper{position:relative}.navbar .column-selector-wrapper .column-selector-box{box-shadow:0 0 10px #d3d3d3;padding:10px;position:absolute;right:0;top:1px;width:150px;z-index:1060}.navbar .column-selector-wrapper .column-selector-box .checkbox{margin-bottom:4px}.navbar .column-selector-wrapper .column-selector-box .column-selector-fixed-column{font-style:italic}"]
            },] }
];
DataTableTitleComponent.ctorParameters = () => [
    { type: DataTableComponent, decorators: [{ type: Inject, args: [forwardRef(() => DataTableComponent),] }] }
];
DataTableTitleComponent.propDecorators = {
    title: [{ type: Input }],
    closeSelector: [{ type: HostListener, args: ['document:click',] }]
};

class MinPipe {
    transform(value, ...args) {
        return Math.min.apply(null, value);
    }
}
MinPipe.decorators = [
    { type: Pipe, args: [{
                name: 'min'
            },] }
];

class PxPipe {
    transform(value, ...args) {
        if (value) {
            return Number(value) >= 0 ? `${value}px` : value;
        }
    }
}
PxPipe.decorators = [
    { type: Pipe, args: [{
                name: 'px'
            },] }
];

class DataTableResource {
    constructor(items) {
        this.items = items;
    }
    query(params, filter) {
        let result = [];
        if (filter) {
            result = this.items.filter(filter);
        }
        else {
            result = this.items.slice(); // shallow copy to use for sorting instead of changing the original
        }
        if (params.sortBy) {
            result.sort((a, b) => {
                if (typeof a[params.sortBy] === 'string') {
                    return a[params.sortBy].localeCompare(b[params.sortBy]);
                }
                else {
                    return a[params.sortBy] - b[params.sortBy];
                }
            });
            if (params.sortAsc === false) {
                result.reverse();
            }
        }
        if (params.offset !== undefined) {
            if (params.limit === undefined) {
                result = result.slice(params.offset, result.length);
            }
            else {
                result = result.slice(params.offset, params.offset + params.limit);
            }
        }
        return new Promise((resolve, reject) => {
            setTimeout(() => resolve(result));
        });
    }
    count() {
        return new Promise((resolve, reject) => {
            setTimeout(() => resolve(this.items.length));
        });
    }
}

class DataTableModule {
}
DataTableModule.decorators = [
    { type: NgModule, args: [{
                imports: [
                    CommonModule,
                    FormsModule
                ],
                declarations: [
                    DataTablePaginationComponent,
                    DataTableRowComponent,
                    DataTableComponent,
                    DataTableTitleComponent,
                    DataTableColumnDirective,
                    DataTableTitleDirective,
                    MinPipe,
                    PxPipe,
                ],
                exports: [
                    DataTableComponent,
                    DataTableRowComponent,
                    DataTableColumnDirective,
                    DataTableTitleComponent,
                    DataTablePaginationComponent,
                    DataTableTitleDirective
                ]
            },] }
];

/**
 * Generated bundle index. Do not edit.
 */

export { DataTableColumnDirective, DataTableComponent, DataTableModule, DataTablePaginationComponent, DataTableResource, DataTableRowComponent, DataTableTitleDirective, defaultTranslations, DataTableTitleComponent as ɵa, MinPipe as ɵb, PxPipe as ɵc };
//# sourceMappingURL=ngx-datatable-bootstrap4.js.map
